"""
Tests that test the functionality of the capability version REST API.
The logic can be found in `capability/views/capability_version.py
"""
import http

from pyramid.config import Configurator
from pyramid.httpexceptions import HTTPBadRequest, HTTPNotFound, HTTPPreconditionFailed
from pyramid.testing import DummyRequest


def test_view_latest_version(test_config: Configurator, request_null_capability: DummyRequest):
    from capability.views.capability_version import view_latest_version

    request_null_capability.matchdict["capability_request_id"] = 1
    response = view_latest_version(request_null_capability)
    expected_json = {
        "type": "CapabilityVersion",
        "capability_request_id": 1,
        "version_number": 1,
        "parameters": '{"arguments": "-pe"}',
        "files": [],
    }
    assert response.status_code == http.HTTPStatus.OK
    assert response.json_body == expected_json

    request_null_capability.matchdict["capability_request_id"] = 2
    no_versions_response = view_latest_version(request_null_capability)
    assert no_versions_response.status_code == http.HTTPStatus.PRECONDITION_FAILED

    request_null_capability.matchdict["capability_request_id"] = 4
    no_such_request_response = view_latest_version(request_null_capability)
    assert no_such_request_response.status_code == http.HTTPStatus.NOT_FOUND


def test_create_capability_version(
    test_config: Configurator, request_null_capability: DummyRequest
):
    """
    Tests the create capability version view to make sure it properly supports creation of versions

    :param test_config: Dummy Pyramid Configurator object set up for testing
    :param request_null_capability: Dummy Pyramid request object set up with mocked DB access
    supporting the null capability
    """
    from capability.views.capability_version import create_capability_version

    request_null_capability.matchdict["capability_request_id"] = 1
    request_null_capability.json_body = {"parameters": "-g"}

    # Assert test capability version not in list of versions (mocked)
    request = request_null_capability.capability_info.lookup_capability_request(1)
    assert len(request.versions) != 3

    response = create_capability_version(request_null_capability)
    assert response.status_code == http.HTTPStatus.OK
    expected_response = {
        "type": "CapabilityVersion",
        "capability_request_id": 1,
        "version_number": 2,
        "parameters": "-g",
    }
    for entry in expected_response:
        assert entry in response.json_body
    # Assert test capability version has been added to list of versions (mocked)
    request = request_null_capability.capability_info.lookup_capability_request(1)
    assert request.versions[1]
